from typing import Dict

from google.protobuf.json_format import MessageToJson, Parse, MessageToDict, ParseDict
from google.protobuf.message import Message
from google.protobuf.struct_pb2 import Value


class ProtoException(ValueError):
    pass


def proto_to_json(obj: Message, use_snake_case: bool = True) -> str:
    if not obj:
        raise ProtoException('Trying to serialize None obj')
    try:
        text = MessageToJson(obj, preserving_proto_field_name=use_snake_case)
    except Exception as e:
        raise ProtoException(f'Error serializing proto message: {e}')
    return text


def proto_to_dict(obj: Message, use_snake_case: bool = True) -> str:
    if not obj:
        raise ProtoException('Trying to serialize None obj')
    try:
        message_dict = MessageToDict(obj, preserving_proto_field_name=use_snake_case)
    except Exception as e:
        raise ProtoException(f'Error converting proto message to dict: {e}')
    return message_dict


def json_to_proto(text: str, proto_clazz, ignore_unknown_fields=True) -> Message:
    if proto_clazz:
        msg = proto_clazz()
    else:
        raise ProtoException('No message class defined')

    try:
        msg = Parse(text, msg, ignore_unknown_fields)
    except Exception as e:
        raise ProtoException(f'Error while parsing text: {e}')
    return msg


def dict_to_proto(d: Dict, proto_clazz, ignore_unknown_fields=True) -> Message:
    if proto_clazz:
        msg = proto_clazz()
    else:
        raise ProtoException('No message class defined')

    try:
        msg = ParseDict(d, msg, ignore_unknown_fields)
    except Exception as e:
        raise ProtoException(f'Error while parsing text: {e}')
    return msg


def get_value(v: Value):
    which_one_of = v.WhichOneof('kind')
    if which_one_of in ('string_value', 'number_value', 'bool_value'):
        return getattr(v, v.WhichOneof('kind'))
    elif which_one_of == 'list_value':
        return [get_value(value) for value in getattr(v, v.WhichOneof('kind')).values]
    return None


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protos/ingestion/schema.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1dprotos/ingestion/schema.proto\x12\x10protos.ingestion\x1a\x1fgoogle/protobuf/timestamp.proto\"\x82\x01\n\x05Value\x12\x16\n\x0cstring_value\x18\x01 \x01(\tH\x00\x12\x14\n\nbool_value\x18\x02 \x01(\x08H\x00\x12\x13\n\tint_value\x18\x03 \x01(\x03H\x00\x12\x16\n\x0c\x64ouble_value\x18\x04 \x01(\x01H\x00\x12\x15\n\x0b\x62ytes_value\x18\x07 \x01(\x0cH\x00\x42\x07\n\x05value\"?\n\x08KeyValue\x12\x0b\n\x03key\x18\x01 \x01(\t\x12&\n\x05value\x18\x02 \x01(\x0b\x32\x17.protos.ingestion.Value\"\x18\n\x08Workflow\x12\x0c\n\x04name\x18\x01 \x01(\t\"\x9d\x01\n\x06WEvent\x12,\n\x08workflow\x18\x01 \x01(\x0b\x32\x1a.protos.ingestion.Workflow\x12-\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\r\n\x05state\x18\x03 \x01(\t\x12\'\n\x03kvs\x18\x04 \x03(\x0b\x32\x1a.protos.ingestion.KeyValue\"5\n\tWEPayload\x12(\n\x06\x65vents\x18\x01 \x03(\x0b\x32\x18.protos.ingestion.WEvent\"F\n\x19WEPayloadIngestionRequest\x12)\n\x04\x64\x61ta\x18\x01 \x01(\x0b\x32\x1b.protos.ingestion.WEPayload\"+\n\x1aWEPayloadIngestionResponse\x12\r\n\x05\x63ount\x18\x01 \x01(\x04\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'protos.ingestion.schema_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _VALUE._serialized_start=85
  _VALUE._serialized_end=215
  _KEYVALUE._serialized_start=217
  _KEYVALUE._serialized_end=280
  _WORKFLOW._serialized_start=282
  _WORKFLOW._serialized_end=306
  _WEVENT._serialized_start=309
  _WEVENT._serialized_end=466
  _WEPAYLOAD._serialized_start=468
  _WEPAYLOAD._serialized_end=521
  _WEPAYLOADINGESTIONREQUEST._serialized_start=523
  _WEPAYLOADINGESTIONREQUEST._serialized_end=593
  _WEPAYLOADINGESTIONRESPONSE._serialized_start=595
  _WEPAYLOADINGESTIONRESPONSE._serialized_end=638
# @@protoc_insertion_point(module_scope)



import json
from unittest import TestCase

from protos.test.test_pb2 import TestMessage
from utils.proto_utils import proto_to_json, json_to_proto


class TestProtoUtils(TestCase):
    def test_proto_to_json(self):
        tm = TestMessage(
            workflow_name='workflow_name',
            workflow_id=1000,
            test_nested_message=TestMessage.TestNestedMessage(
                string_field='string', bool_field=True
            )
        )

        snake_case = """
        {
          "workflow_name": "workflow_name",
          "workflow_id": "1000",
          "test_nested_message": {
            "string_field": "string",
            "bool_field": true
          }
        }
        """
        self.assertEqual(proto_to_json(tm), json.dumps(json.loads(snake_case), indent=2))

        camel_case = """
        {
          "workflowName": "workflow_name",
          "workflowId": "1000",
          "testNestedMessage": {
            "stringField": "string",
            "boolField": true
          }
        }
        """
        self.assertEqual(proto_to_json(tm, use_snake_case=False), json.dumps(json.loads(camel_case), indent=2))

    def test_json_to_proto(self):
        expected_out = tm = TestMessage(
            workflow_name='workflow_name',
            workflow_id=1000,
            test_nested_message=TestMessage.TestNestedMessage(
                string_field='string', bool_field=True
            )
        )

        snake_case = """
        {
          "workflow_name": "workflow_name",
          "workflow_id": "1000",
          "test_nested_message": {
            "string_field": "string",
            "bool_field": true
          }
        }
        """

        self.assertEqual(json_to_proto(snake_case, TestMessage), expected_out)

        camel_case = """
        {
          "workflowName": "workflow_name",
          "workflowId": "1000",
          "testNestedMessage": {
            "stringField": "string",
            "boolField": true
          }
        }
        """
        self.assertEqual(json_to_proto(camel_case, TestMessage), expected_out)




import time
import datetime
import json
import requests
import os

from pydoctordroid.protos.ingestion import schema_pb2 as schema__pb2
from pydoctordroid.utils.proto_utils import proto_to_json

class DroidEvents():

	def __init__(self):
		self.setup = False
		self.org_name = os.environ.get('ORG_NAME')
		self.hostname = os.environ.get('DRDROID_HOSTNAME')
		if self.org_name and self.hostname:
			self.setup = True
			self.PUBLISH_ENDPOINT = "{}/w/agent/push_events".format(self.hostname)
			self.headers = {'Content-Type': 'application/json', 'X-REQUEST-ORG': self.org_name}


	def assign_value(self, value_object, value_passed):
		value_passed_type = type(value_passed)

		if value_passed_type == int:
			value_object.int_value = value_passed

		elif value_passed_type == float:
			value_object.double_value = value_passed

		elif value_passed_type == bool:
			value_object.bool_value = value_passed

		elif value_passed_type == str:
			value_object.string_value = value_passed

		elif value_passed_type == bytes:
			value_object.bytes_value = value_passed

		else:
			value_object.string_value = str(value_passed)


	def publish(self, wf_name, wf_state, kv_pairs, event_time=None):

		if not self.setup:
			return

		payloadRequest = schema__pb2.WEPayloadIngestionRequest()
		wePayload = payloadRequest.data
		wEvent = wePayload.events.add()

		wEvent.state = wf_state

		workflow = wEvent.workflow
		workflow.name = wf_name

		if kv_pairs:
			for kv in kv_pairs:
				_kv = wEvent.kvs.add()
				_kv.key = kv[0]
				_v = _kv.value
				self.assign_value(_v, kv[1])

		timestamp = wEvent.timestamp

		# Take event_time passed, else use current time. Use current time in case of an exception
		try:
			if event_time:
				timestamp.seconds = int(event_time)
			else:
				timestamp.seconds = int(time.time())
		except:
			timestamp.seconds = int(time.time())

		try:
			self.send_http(payloadRequest)
		except:
			pass


	def send_http(self, proto_payload_request):
		payload_string = proto_to_json(proto_payload_request)
		# print(payload_string)
		resp = requests.post(self.PUBLISH_ENDPOINT, headers=self.headers, data=payload_string)
		# print("Send HTTP Status Code: {}".format(resp.status_code))
		# print("Send HTTP Body: {}".format(resp.text))

